#include <string.h>
#include <proto.h>
#include "dialog.h"

void guiid_required(){}	// To ease the link
static int alloc;

class GUI_RESOURCE: public ARRAY_OBJ{
public:
	char id[10];
	/*~PROTOBEG~ GUI_RESOURCE */
public:
	GUI_RESOURCE (void);
	/*~PROTOEND~ GUI_RESOURCE */
};


PUBLIC GUI_RESOURCE::GUI_RESOURCE()
{
	sprintf (id,"R%d",alloc++);
}

class FONT: public GUI_RESOURCE{
public:
	int pointsize;
	int fontid;
	int style;
	int weight;
	bool underlined;
	/*~PROTOBEG~ FONT */
public:
	FONT (int _pointsize,
		 int _fontid,
		 int _style,
		 int _weight,
		 bool _underlined);
	/*~PROTOEND~ FONT */
};

PUBLIC FONT::FONT(
	int _pointsize,
	int _fontid,
	int _style,
	int _weight,
	bool _underlined)
{
	pointsize = _pointsize;
	fontid = _fontid;
	style = _style;
	weight = _weight;
	underlined = _underlined;
	diagui_sendcmd (P_Deffont,"%s %d %d %d %d %d\n",id,_pointsize,_fontid
		,_style,_weight,_underlined);
}

class FONTS: public ARRAY{
	/*~PROTOBEG~ FONTS */
public:
	FONT *getitem (int no)const;
	/*~PROTOEND~ FONTS */
};
PUBLIC FONT *FONTS::getitem (int no) const
{
	return (FONT*)ARRAY::getitem(no);
}

static FONTS fonts;
/*
	Return the ID describing a specific font
*/
EXPORT const char *guiid_setfont (
	int pointsize,
	int fontid,
	int style,
	int weight,
	bool underlined)
{
	const char *ret = NULL;
	// Locate an equivalent font, so we avoid transmitting a font spec
	// for nothing.
	for (int i=0; i<fonts.getnb(); i++){
		FONT *f = fonts.getitem(i);
		if (f->pointsize == pointsize
			&& f->fontid == fontid
			&& f->style == style
			&& f->weight == weight
			&& f->underlined == underlined){
			ret = f->id;
			break;
		}
	}
	if (ret == NULL){
		FONT *f = new FONT (pointsize,fontid,style,weight,underlined);
		fonts.add (f);
		ret = f->id;
	}
	return ret;
}

class PEN: public GUI_RESOURCE{
public:
	char color[20];
	int thick;
	int style;
	/*~PROTOBEG~ PEN */
public:
	PEN (const char *_color, int _thick, int _style);
	/*~PROTOEND~ PEN */
};

PUBLIC PEN::PEN(const char *_color, int _thick, int _style)
{
	strcpy_cut (color,_color,sizeof(color)-1);
	thick = _thick;
	style = _style;
	diagui_sendcmd (P_Defpen,"%s %s %d %d\n",id,_color,_thick,_style);
}

class PENS: public ARRAY{
	/*~PROTOBEG~ PENS */
public:
	PEN *getitem (int no)const;
	/*~PROTOEND~ PENS */
};
PUBLIC PEN *PENS::getitem (int no) const
{
	return (PEN*)ARRAY::getitem(no);
}
static PENS pens;
/*
	Return the ID describing a specific pen
*/
EXPORT const char *guiid_setpen (
	const char *color,
	int thick,
	int style)
{
	const char *ret = NULL;
	for (int i=0; i<pens.getnb(); i++){
		PEN *p = pens.getitem(i);
		if (strcmp(p->color,color)==0
			&& p->thick == thick
			&& p->style == style){
			ret = p->id;
			break;
		}
	}
	if (ret == NULL){
		PEN *p = new PEN (color,thick,style);
		pens.add (p);
		ret = p->id;
	}
	return ret;
}

class BRUSH: public GUI_RESOURCE{
public:
	char color[20];
	int style;
	/*~PROTOBEG~ BRUSH */
public:
	BRUSH (const char *_color, int _style);
	/*~PROTOEND~ BRUSH */
};

PUBLIC BRUSH::BRUSH(const char *_color, int _style)
{
	strcpy_cut (color,_color,sizeof(color)-1);
	style = _style;
	diagui_sendcmd (P_Defbrush,"%s %s %d\n",id,_color,_style);
}

class BRUSHES: public ARRAY{
	/*~PROTOBEG~ BRUSHES */
public:
	BRUSH *getitem (int no)const;
	/*~PROTOEND~ BRUSHES */
};
PUBLIC BRUSH *BRUSHES::getitem (int no) const
{
	return (BRUSH*)ARRAY::getitem(no);
}

static BRUSHES brushes;

/*
	Return the ID describing a specific brush
*/
EXPORT const char *guiid_setbrush (
	const char *color,
	int style)
{
	const char *ret = NULL;
	for (int i=0; i<brushes.getnb(); i++){
		BRUSH *p = brushes.getitem(i);
		if (strcmp(p->color,color)==0
			&& p->style == style){
			ret = p->id;
			break;
		}
	}
	if (ret == NULL){
		BRUSH *p = new BRUSH (color,style);
		brushes.add (p);
		ret = p->id;
	}
	return ret;
}

class GUI_RESOURCES: public ARRAY{
	/*~PROTOBEG~ GUI_RESOURCES */
	/*~PROTOEND~ GUI_RESOURCES */
};

static GUI_RESOURCES dcs;

EXPORT const char *guiid_setdc (
	const char *font,
	const char *pen,
	const char *brush)
{
	if (font == NULL) font = guiid_setfont (12,0,0,0,false);
	if (pen == NULL) pen = guiid_setpen ("black",0,0);
	if (brush == NULL) brush = guiid_setbrush ("black",0);
	GUI_RESOURCE *r = new GUI_RESOURCE;
	dcs.add (r);
	diagui_sendcmd (P_Defdc,"%s $font=%s pen=%s brush=%s\n"
		,r->id,font,pen,brush);
	return r->id;
}

