/* ***********

	Check comments starting with // ### and fill appropriate code there

   ***********
*/
#pragma implementation
#include <stdio.h>
#include <string.h>
#include <translat.h>
#include <netconf.h>
#include <userconf.h>
#include "redhat.h"
#include "redhat.m"
#include "../module_apis/package_api.h"
#include "../module_apis/servicectl_api.h"

MODULE_DEFINE_VERSION(redhat);

static HELP_FILE help_installpkg ("redhat","installpkg");

PUBLIC MODULE_redhat::MODULE_redhat()
	: LINUXCONF_MODULE ("redhat")
{
	linuxconf_loadmsg ("redhat",PACKAGE_REV);
	linuxconf_sethook (filter_fctnew);
	int release = redhat_release();
	if (release>=501){
		hostinfo_sethook (devinfo_load,devinfo_save);
		groutes_sethook (routes_load,routes_save);
		ipalias_sethook (devinfo_ipalias_load,devinfo_ipalias_save);
	}
	if (release>=500){
		#if defined(__GLIBC__) && defined(HAS_PAM)
			users_sethook (reduser_del,reduser_add,reduser_change);
			passwd_sethook (pam_check_pass,pam_change_pass,pam_check_pair);
		#else
			xconf_notice (MSG_U(N_WRONGVERSION
				,"You have installed a Linuxconf version suited for\n"
				 "RedHat 4.x on a RedHat 5.x system.\n"
				 "Things will operate normally, except for the following\n"
				 "shortcomings:\n"
				 "    -PAM won't be used for user authentication and passwords\n"
				 "     updates.\n"
				 "    -Linuxconf will update the password database itself\n"
				 "     instead of relying on the useradd, userdel and usermod\n"
				 "     utilities.\n"
				 "We suggest you install the proper version. You can \"upgrade\"\n"
				 "by using the -U and --force rpm command line options."));
		#endif
	}
	module_register_api (SERVICECTL_API_KEY,SERVICECTL_API_REV
		,sysv_api_get,sysv_api_release);
}


PUBLIC int MODULE_redhat::dohtml (const char *key)
{
	int ret = LNCF_NOT_APPLICABLE;
	if (strcmp(key,"redhat")==0){
		// ### Insert any menu and dialog here
		ret = 0;
	}
	return ret;
}


static void usage()
{
	xconf_error (MSG_U(T_USAGE
		,"Module redhat\n"
		 "linuxconf --modulemain redhat [ specific options ]\n"
		 "\n")
		);
}

PUBLIC int MODULE_redhat::execmain (int argc , char *argv[], bool)
{
	int ret = LNCF_NOT_APPLICABLE;
	const char *pt = strrchr(argv[0],'/');
	if (pt != NULL){
		pt++;
	}else{
		pt = argv[0];
	}
	if (strcmp(pt,"redhat")==0){
		ret = -1;
		if (argc == 1){
			// ### Place call to main menu of the module
		}else{
			// ### Add some option parsing for the module
			::usage();
		}
	}
	return ret;
}

static const char K_CHECK_PACKAGE[]="check_package";

PUBLIC int MODULE_redhat::message (const char *msg, int argc , const char *argv[])
{
	int ret = LNCF_NOT_APPLICABLE;
	if (strcmp(msg,"check_package")==0){
		ret = 0;
		const char *logical_name = argv[0];
		if (linuxconf_getvalnum (K_CHECK_PACKAGE,logical_name,0)==0){
			// Ok, this is the first time we do this test
			// We probably need one table per redhat version or something
			// like that.
			static const char *tblk[][3]={
				// Canonical name,	RedHat name, description
				{"dnsserver",	"bind",	MSG_U(I_BINDDESC,"Domain name server (DNS)")},
				{"popserver",	"imap",	MSG_U(I_IMAPDESC,"POP-3 and IMAP mail servers")},
				{"dhcpserver",	"dhcp",	MSG_U(I_DHCPDESC,"Dynamic host configuration protocol server (DHCP)")},
				{"nfsserver",	"nfs-utils",MSG_U(I_NFSDESC,"Network file system server (NFS)")},
				{NULL,NULL,NULL},
			};
			// Check if it is a package we know
			for (int i=0; tblk[i][0] != NULL; i++){
				if (strcmp(tblk[i][0],logical_name)==0){
					const char *rhpkg = tblk[i][1];
					PACKAGE_API *pkg_api = package_api_init ("redhat/redhat.cc");
					if (pkg_api != NULL){
						PACKAGE_VERSION res;
						if (pkg_api->getver (pkg_api, rhpkg, res)==-1){
							// Ok time to install
							SSTRING q;
							q.setfromf (MSG_U(Q_INSTALLPKG
								,"The following package is not currently installed\n"
								 "\n"
								 "\t%s: %s\n"
								 "\n"
								 "would you like to install it now ?")
								,rhpkg,tblk[i][2]);
							if (xconf_yesno (MSG_U(T_INSTALLPKG,"Suggested package")
								,q.get(),help_installpkg)==MENU_YES
								&& perm_rootaccess (MSG_U(P_INSTALLPKG,"install packages"))){	
								SSTRINGS tb;
								tb.add (new SSTRING(rhpkg));
								pkg_api->installpkgs (pkg_api,tb,"/mnt/cdrom/RedHat/RPMS");
							}
						}
						package_api_end (pkg_api);
					}
					break;
				}
			}
			// Make sure we do not ask this twice to the user
			linuxconf_replace (K_CHECK_PACKAGE,logical_name,1);
			perm_setbypass (true);
			linuxconf_save();
			perm_setbypass (false);
		}
	}
	return ret;
}
		
	


static MODULE_redhat redhat;

const char *vendor_getid()
{
	return "Red Hat Software";
}

const char *vendor_getdistid()
{
	int release = redhat_release();
	const char *ret = "";
	if (release == 601){
		ret = "BlueSky";
	}else if (release == 600){
		ret = "BlueSky";
	}else if (release == 502){
		ret = "Manhattan";
	}else if (release == 501){
		ret = "Manhattan";
	}else if (release == 500){
		ret = "Hurricane";
	}else if (release == 402){
		ret = "Biltmore";
	}
	return ret;
}

